class CSharpDict:
    Keys: list[str]
    Values: list[str]


class DataViewQueryTranslationResult:
    DaxExpression: str
    SelectNameToDaxColumnName: CSharpDict


def prototype_query(query: str, db_name: str, port: int) -> "DataViewQueryTranslationResult":
    ...


class CSharpDict:
    """The standard C# dictionary type."""

    Keys: list[str]
    """The keys of the dictionary as a list."""

    Values: list[str]
    """The values of the dictionary as a list."""

class DataViewQueryTranslationResult:
    """Result of the query translation."""

    DaxExpression: str
    """The translated DAX expression for the given SSAS instance."""

    SelectNameToDaxColumnName: CSharpDict
    """Mapping from the names of the columns in the SELECT statement to the DAX column names.
    
    Example:
    
        ```json
        {'Sum(Sales.Sales Amount)': '[SumSales_Amount]', 'Sales.Sales Amount by Due Date': '[Sales_Amount_by_Due_Date]', 'Date.Fiscal.Month': 'Date[Month]'}
        ```
    """

class PrototypeQuery:
    """Object representing a query prototype on the C# side."""

    @staticmethod
    def Translate(
        query: str, dbName: str, port: int, workingDirectory: str | None = None
    ) -> DataViewQueryTranslationResult: ...
    """Translates a query to DAX expression."""



from pathlib import Path
import shutil

root = Path(__file__)
input_folder = root.parents[1] / 'translation/bin/x64/Debug'
output_folder = root.parent / 'pbi_prototype_query_translation/libs'
output_folder.mkdir(parents=True, exist_ok=True)
# (output_folder / '__init__.py').write_text("# Just here to remind hatchling that we want this too")

for f in output_folder.iterdir():
    f.unlink()

for f in input_folder.iterdir():
    if f.is_dir():
        continue
    if f.name in (
        'Microsoft.PowerBI.ClientResources.dll', 
        'Microsoft.Mashup.Client.Desktop.UI.dll',
        'Microsoft.PowerBI.Client.Windows.Themes.dll',
        'msmdrsv.exe'
    ):
        continue
    shutil.copy(
        f.absolute().as_posix(),
        (output_folder / f.name).absolute().as_posix()  
    )


import sys
from pathlib import Path
from typing import TYPE_CHECKING

import clr  # type: ignore[import-untyped]

SOURCE_FOLDER = (Path(__file__).parent / "libs").absolute().as_posix()
sys.path.insert(0, SOURCE_FOLDER)
clr.AddReference("Translation")  # type: ignore
# When type checking, we import from the stub file. When running, we import from the actual C# module.
if TYPE_CHECKING:
    from .Translation import (  # type: ignore[import-untyped]
        DataViewQueryTranslationResult,
        PrototypeQuery,
    )
else:
    from Translation import DataViewQueryTranslationResult, PrototypeQuery


class TranslationResult:
    """Result of the query translation, converted to Python structures."""

    dax: str
    """The translated DAX expression for the given SSAS instance."""

    column_mapping: dict[str, str]
    """Mapping from the names of the columns in the SELECT statement to the DAX column names.
    
    Example:
    
        ```json
        {'Sum(Sales.Sales Amount)': '[SumSales_Amount]', 'Sales.Sales Amount by Due Date': '[Sales_Amount_by_Due_Date]', 'Date.Fiscal.Month': 'Date[Month]'}
        ```
    """

    def __init__(self, data: DataViewQueryTranslationResult):
        self.data = data
        self.dax = data.DaxExpression
        self.column_mapping = dict(
            zip(
                data.SelectNameToDaxColumnName.Keys,
                data.SelectNameToDaxColumnName.Values,
            )
        )


def prototype_query(query: str, db_name: str, port: int) -> TranslationResult:
    """Main entrypoint for this library."""
    ret = PrototypeQuery.Translate(query, db_name, port, SOURCE_FOLDER)
    return TranslationResult(ret)
